/*
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDEVELOPER, TCSDESIGNER
 * Version: 1.0
 */

using System;
using System.IO;
using TopCoder.Util.CommandLine;
using TopCoder.Util.ConfigurationManager;
using System.Collections;

namespace TopCoder.File.Statistics
{
    /// <summary>
    /// This class is a command line interface to the component.
    /// </summary>
    /// <remarks>
    /// The following options are supported:
    /// <list>
    /// <item>
    /// /c:config-file
    /// A config file that's initialized (otherwise based on preload for configuration manager).
    /// </item>
    /// <item>
    /// /n:namespace Provides a namespace to initialize from. If neither is provided the default namespace is used.
    /// That namespace should be preloaded.
    /// </item>
    /// <item>
    /// /b:base-path
    /// All paths are relative to this base.
    /// </item>
    /// <item>
    /// /r
    /// Enables recursive descent through sub-directories.
    /// </item>
    /// <item>
    /// /o:output-file
    /// An output filename. If not specified, output goes to console.
    /// </item>
    /// </list>
    /// </remarks>
    public class CommandLine
    {
        /// <summary>
        /// This is the default namespace used if none is specifically defined in the command line arguments.
        /// </summary>
        public const string Namespace = "TopCoder.File.Statistics.CommandLine";

        /// <summary>
        /// The help message that's shown if the command is called without options or with invalid options.
        /// </summary>
        private const string HelpMessage =
            @"Generates a file statistics report.

Usage: {0} [/c:config-file] [/n:namespace] [/b:base-path] [/r] [/o] path [path2 path3 ...]

  /c:config-file    A config file to load.
  /n:namespace      Provides a namespace to initialize from. If neither of these are provided the default namespace is
                    used. That namespace should be preloaded.
  /b:base-path      All paths are relative to this base.
  /r                Enabls recursive descent through sub-directories.
  /o:output-file    An output filename. If not specified, output goes to console.
  paths             The paths to survey.

";

        /// <summary>
        /// A constructor to stop from instantiation of this class, as that isn't intended.
        /// </summary>
        private CommandLine()
        {
        }

        /// <summary>
        /// Prints the help message, with the correct file name inserted.
        /// </summary>
        public static void PrintHelp()
        {
            Console.Error.WriteLine(HelpMessage, typeof(CommandLine).Assembly.Location);
        }

        /// <summary>
        /// Command line entrypoint. The class documentation details the available options.
        /// </summary>
        /// <param name="args"></param>
        public static void Main(string[] args)
        {
            // If no arguments, print help and quit
            if (args.Length == 0)
            {
                PrintHelp();
                return;
            }

            // Prepare the config fields.
            // They are not thread-safe, we must be careful and not share them if this class should be called
            // programmatically.
            CommandLineSwitch configFileOption = new CommandLineSwitch("c");
            configFileOption.Factor = (int) FactorParams.ONE_OR_MORE;

            CommandLineSwitch namespaceOption = new CommandLineSwitch("n");
            configFileOption.Factor = 1;

            CommandLineSwitch basePathOption = new CommandLineSwitch("b");
            basePathOption.Factor = 1;

            CommandLineSwitch recursiveOption = new CommandLineSwitch("r");
            recursiveOption.Factor = 0;

            CommandLineSwitch outputFileOption = new CommandLineSwitch("o");
            recursiveOption.Factor = 1;

            CommandLineParser parser = new CommandLineParser(new CommandLineSwitch[]
                {
                    configFileOption,
                    namespaceOption,
                    basePathOption,
                    recursiveOption,
                    outputFileOption
                }, true);

            // This is the statistics engine object. It isn't actually set until the switches have been parsed.
            ExtendedFileStatistics statEngine;

            // Parsing and some reading is done here. Error in this part results in a help message.
            string basePath = "";
            try
            {
                parser.Parse(args);
                if (configFileOption.Result != null)
                {
                    foreach (string configFile in configFileOption.Result)
                    {
                        ConfigManager.GetInstance().LoadFile(configFile);
                    }
                }

                string nameSpace = Namespace;
                if (namespaceOption.Result != null)
                {
                    nameSpace = (string) namespaceOption.Result[0];
                }

                statEngine = new ExtendedFileStatistics(nameSpace);

                if (basePathOption.Result != null)
                {
                    statEngine.Reporting.Reset(new DirectoryInfo((string) basePathOption.Result[0]));
                    basePath = (string) basePathOption.Result[0];
                }
            }
            catch (Exception e)
            {
                Console.Error.WriteLine(e);
                Console.Error.WriteLine();
                PrintHelp();
                return;
            }

            // These are errors while running, help doesn't make much sense.
            try
            {
                bool recursive = (parser.InputtedSwitches["r"] != null);
                foreach (string str in parser.Arguments)
                {
                    string realStr = Path.Combine(basePath, str);

                    FileInfo file = new FileInfo(realStr);
                    if (file.Exists)
                    {
                        statEngine.ProcessFile(file);
                    }
                    else
                    {
                        DirectoryInfo directory = new DirectoryInfo(realStr);
                        statEngine.ProcessDirectory(directory, recursive);
                    }
                }

                if (outputFileOption.Result != null)
                {
                    statEngine.PrintReport(new FileInfo((string) outputFileOption.Result[0]));
                }
                else
                {
                    statEngine.PrintReport();
                }
            }
            catch (Exception e)
            {
                Console.Error.WriteLine(e);
                Console.Error.WriteLine();
            }
        }
    }
}