/*
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDEVELOPER, TCSDESIGNER
 * Version: 1.0
 */


using System;
using System.Collections;
using System.Collections.Specialized;
using System.IO;
using TopCoder.Util.ConfigurationManager;

namespace TopCoder.File.Statistics
{
    /// <summary>
    /// This is an extended version of <c>FileStatistics</c> which supports some advanced operations including
    /// directory processing and reporting printing. The reason to separate this out is that we may not need to rely on
    /// the directory traversing algorithm incorporated here (which is recursive).
    /// </summary>
    /// <remarks>
    /// This class shares the same configuration logic as <c>FileStatistics</c>. The constructor simply propagates the
    /// namespace to the super class.
    ///
    /// All the operations are delegated. This class does not have any states and is fully dependant on
    /// <c>FileStatistics</c> threading level, which is no threading safety.
    /// </remarks>
    public class ExtendedFileStatistics : FileStatistics
    {
        /// <summary>
        /// Default constructor. Does nothing.
        /// </summary>
        public ExtendedFileStatistics()
            : base()
        {
        }

        /// <summary>
        /// Constructor with namespace. Simply propagate to super constructor.
        /// </summary>
        /// <param name="nameSpace">The namespace to load the config from.</param>
        /// <exception cref="ArgumentNullException">If <c>namespace</c> is null.</exception>
        /// <exception cref="ArgumentException">If <c>namespace</c> is empty, when trimmed.</exception>
        /// <exception cref="ConfigurationException">If a config error occurs.</exception>
        public ExtendedFileStatistics(string nameSpace)
            : base(nameSpace)
        {
        }

        /// <summary>
        /// Process a directory in non-recursive mode. Simply calls <c>ProcessDirectory(directory, false)</c>.
        /// </summary>
        /// <param name="directory">The directory to process.</param>
        /// <exception cref="ArgumentNullException">If <c>directory</c> is null.</exception>
        /// <exception cref="IOException">If any IO error occurs.</exception>
        /// <exception cref="FileStatisticsException">If processing fails.</exception>
        public void ProcessDirectory(DirectoryInfo directory)
        {
            ProcessDirectory(directory, false);
        }

        /// <summary>
        /// Process a directory in given mode. If it's non-recursive only the files in the directory is processed,
        /// otherwise every file in every sub-directory is processed.
        /// </summary>
        /// <param name="directory">The directory to process.</param>
        /// <param name="recursive">If the method should call itself recursively for every subdirectory.</param>
        /// <exception cref="ArgumentNullException">If <c>directory</c> is null.</exception>
        /// <exception cref="IOException">If any IO error occurs.</exception>
        /// <exception cref="FileStatisticsException">If processing fails.</exception>
        public void ProcessDirectory(DirectoryInfo directory, bool recursive)
        {
            if (directory == null)
            {
                throw new ArgumentNullException("directory");
            }

            foreach (FileInfo file in directory.GetFiles())
            {
                ProcessFile(file);
            }

            if (recursive)
            {
                foreach (DirectoryInfo subdirectory in directory.GetDirectories())
                {
                    ProcessDirectory(subdirectory, true);
                }
            }
        }

        /// <summary>
        /// Prints a report on the console.
        /// </summary>
        public void PrintReport()
        {
            Console.WriteLine(Reporting.GenerateReport());
        }

        /// <summary>
        /// Prints a report to the given file.
        /// </summary>
        /// <param name="file">The file to write to.</param>
        /// <exception cref="ArgumentNullException">If <c>file</c> is null.</exception>
        /// <exception cref="IOException">If any IO error occurs.</exception>
        /// <exception cref="StatsReportingException">If rendering fails.</exception>
        public void PrintReport(FileInfo file)
        {
            if (file == null)
            {
                throw new ArgumentNullException("file");
            }

            // This encoding is used so that a stream resulting from the XmlStatsReporting has the correct encoding
            // data. Otherwise, we would have to actually examine the results to see if it seemed to be XML and, if so,
            // manipulate the encoding.
            using (TextWriter stream = new StreamWriter(file.FullName, false, System.Text.Encoding.Unicode))
            {
                stream.WriteLine(Reporting.GenerateReport());
            }
        }
    }
}
