/*
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDEVELOPER, TCSDESIGNER
 * Version: 1.0
 */


using System;
using System.Collections;
using System.IO;

namespace TopCoder.File.Statistics
{
    /// <summary>
    /// <c>IFileProcessor</c> defines the contract to process a fail. A file processor defines a method for producing
    /// statistical results for a certain class of files, regarding certain metrics. (For instance count line numbers
    /// for a text file.)
    ///
    /// Multiple IFileProcessors are aggregated by <c>FileStatistics</c>, which can run in two modes (process with
    /// first matching processor and process with each matching processor, respectively). File processors are mapped
    /// with an alias. This alias is dynamically provided by the user and the reporting can be grouped against this
    /// alias.
    /// </summary>
    /// <remarks>
    /// Implementations are advised to be stateless and thread-safe. But it's not a requirement. Also, if an
    /// implementation needs to be created from an XML configuration, it should have a public constructor accepting
    /// a Configuration Manager namespace and property prefix. Suffixes are appended to this prefix to retrieve
    /// relevant configuration for the component.
    /// </remarks>
    public interface IFileProcessor
    {
        /// <summary>
        /// Get the alias of the processor. An alias is the official name of the processor which is also used when
        /// classifying the stat results.
        /// </summary>
        /// <value>
        /// A non-null, non-empty string, normally constant over the lifetime of a specific processor.
        /// </value>
        string Alias
        {
            get;
        }

        /// <summary>
        /// Get the file matcher associated with the processor in order to determine whether a file can/should be
        /// processed with the processor.
        /// </summary>
        /// <value>
        /// An <c>IFileMatcher</c> implementation. The <c>FileStatistics</c> class is accessing this property to match
        /// files before sending them to the processor itself. Never null.
        /// </value>
        IFileMatcher FileMatcher
        {
            get;
        }

        /// <summary>
        /// Get a list of all the supported metrics by this processor. This method is mostly for end user. It is
        /// currently not used by the framework logic in any way.
        /// </summary>
        /// <value>
        /// An <c>IList</c> of <c>Metric</c> instances.
        /// </value>
        IList SupportedMetrics
        {
            get;
        }

        /// <summary>
        /// Process the file and generate the result. A new result should be generated each time, where the keys are
        /// the metrics in <c>SupportedMetrics</c>.
        /// </summary>
        /// <param name="file"></param>
        /// <returns></returns>
        StatsResult Process(FileInfo file);
    }
}