/*
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDEVELOPER, TCSDESIGNER
 * Version: 1.0
 */

using System;
using System.Collections;
using System.Globalization;
using System.IO;
using TopCoder.Util.ConfigurationManager;
using TopCoder.Unix.MagicNumbers;

namespace TopCoder.File.Statistics.Matcher
{
    /// <summary>
    /// An implementation of <c>IFileMatcher</c> that matches one or more specific file name extensions.
    /// <see cref="FileNameMatcher" /> for information about the abstract base class that simplifies operations.
    /// </summary>
    /// <remarks>
    /// The implementation is stateless and thread-safe. The contract constructor for instantiation from Configuration
    /// Manager persistence is supported, with a subproperty name of "extensions". The extension comparison is case
    /// insensitive.
    /// </remarks>
    public class ExtensionMatcher : FileNameMatcher
    {
        /// <summary>
        /// The name of the "subproperty", where the name is appended to the main property name.
        /// </summary>
        public string TypesPropertyKey = ".extensions";

        /// <summary>
        /// A map of the extension strings to validate against. This is done in a case-insensitive manner. The map is
        /// initialized in the constructors. The value is really just used as a set. The point is to achieve faster
        /// comparisons, and better scalability, than by just using a list approach.
        /// </summary>
        private readonly IDictionary extensions;

        /// <summary>
        /// Constructor taking a single extension.
        /// </summary>
        /// <param name="extension">The file extension (excluding period). Validated case-insensitively.</param>
        /// <exception cref="ArgumentNullException">If <c>extension</c> is null.</exception>
        public ExtensionMatcher(string extension)
        {
            if (extension == null)
            {
                throw new ArgumentNullException("extension");
            }

            extensions = Helper.CreateDictionary(1, false);
            // The dictionary is only used as a set.
            extensions.Add(extension, null);
        }

        /// <summary>
        /// Constructor taking a list of type names.
        /// </summary>
        /// <param name="extensions">A list of type names to clone and store internally.</param>
        /// <exception cref="ArgumentNullException">If <c>types</c> is null.</exception>
        /// <exception cref="ArgumentException">If <c>types</c> is empty or contains non-string elements.</exception>
        public ExtensionMatcher(IList extensions)
        {
            this.extensions = Helper.PopulateFromList(extensions, "extensions", false, true);
        }

        /// <summary>
        /// The contract constructor for creating <c>IFileMatcher</c> instances from the configuration manager. This is
        /// done by reading relevant simulated "sub-properties" under <c>prefix</c>.
        /// </summary>
        /// <remarks>
        /// A list of type names is expected to be stored under prefix + <c>TypesPropertyKey</c>, as a multi-valued
        /// property of strings.
        /// </remarks>
        /// <param name="nameSpace">The namespace of the property.</param>
        /// <param name="prefix">The prefix to the property names.</param>
        /// <exception cref="ArgumentNullException">If <c>nameSpace</c> or <c>prefix</c> is null.</exception>
        /// <exception cref="ConfigurationException">If the namespace or property is not found, or if they exist, but
        /// the value list is empty or contains empty strings.</exception>
        public ExtensionMatcher(string nameSpace, string prefix)
        {
            this.extensions = Helper.PopulateFromConfiguration(nameSpace, prefix, TypesPropertyKey, false, true);
        }

        /// <summary>
        /// Implementation of the abstract method in base class, matches if the extension of the specified file name
        /// matches any of the extensions defined by the constructor, with case insensitive comparison (in the
        /// invariant culture).
        /// </summary>
        /// <param name="filename">The file name to match.</param>
        /// <exception cref="FileMatchingException">Any error condition.</exception>
        /// <returns>True if the file is matched, false otherwise.</returns>
        protected override bool Match(string filename)
        {
            string extension = Path.GetExtension(filename);
            if (extension.Length > 0 && extension[0] == '.')
            {
                extension = extension.Substring(1);
            }

            return extensions.Contains(extension);
        }
    }
}