/*
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDEVELOPER, TCSDESIGNER
 * Version: 1.0
 */

using System;
using System.Collections;
using System.Globalization;
using System.IO;
using TopCoder.Util.ConfigurationManager;
using TopCoder.Unix.MagicNumbers;

namespace TopCoder.File.Statistics.Matcher
{
    /// <summary>
    /// An implementation of <c>IFileMatcher</c> that matches one or more specific MIME type.
    /// <see cref="FileTypeMatcher" /> for information on how the file type is determined.
    /// </summary>
    /// <remarks>
    /// The implementation is stateless and thread-safe. The contract constructor for instantiation from Configuration
    /// Manager persistence is supported, with a subproperty name of "types".
    /// </remarks>
    public class MimeMatcher : FileTypeMatcher
    {
        /// <summary>
        /// The name of the "subproperty", where the name is appended to the main property name.
        /// </summary>
        public string MimesPropertyKey = ".mimes";

        /// <summary>
        /// A map of the mime strings to validate against. This is done in a case-sensitive manner. The map is
        /// initialized in the constructors. The value is really just used as a set. The point is to achieve faster
        /// comparisons with better scalability than by just using a list approach.
        /// </summary>
        private readonly IDictionary mimes;

        /// <summary>
        /// Constructor taking a single mime type name.
        /// </summary>
        /// <param name="mime">The mime type name. Validated case-sensitively.</param>
        /// <exception cref="ArgumentNullException">If <c>type</c> is null.</exception>
        /// <exception cref="ArgumentException">If <c>type</c> (trimmed) is empty.</exception>
        public MimeMatcher(string mime)
        {
            if (mime == null)
            {
                throw new ArgumentNullException("mime");
            }
            if (mime.Trim().Length == 0)
            {
                throw new ArgumentException("Empty type string specified.", "mime");
            }

            mimes = Helper.CreateDictionary(1, true);
            // The dictionary is only used as a set.
            mimes.Add(mime, null);
        }

        /// <summary>
        /// Constructor taking a list of mime type names.
        /// </summary>
        /// <param name="mimes">A list of mime type names to clone and store internally.</param>
        /// <exception cref="ArgumentNullException">If <c>types</c> is null.</exception>
        /// <exception cref="ArgumentException">If <c>types</c> is empty, or contains non-string elements or string
        /// elements that are empty when trimmed.</exception>
        public MimeMatcher(IList mimes)
        {
            this.mimes = Helper.PopulateFromList(mimes, "mimes", true, false);
        }

        /// <summary>
        /// The contract constructor for creating <c>IFileMatcher</c> instances from the configuration manager. This is
        /// done by reading relevant simulated "sub-properties" under <c>prefix</c>.
        /// </summary>
        /// <remarks>
        /// A list of mime type names is expected to be stored under prefix + <c>MimesPropertyKey</c>, as a
        /// multi-valued property of strings.
        /// </remarks>
        /// <param name="nameSpace">The namespace of the property.</param>
        /// <param name="prefix">The prefix to the property names.</param>
        /// <exception cref="ArgumentNullException">If <c>nameSpace</c> or <c>prefix</c> is null.</exception>
        /// <exception cref="ConfigurationException">If the namespace or property is not found, or if they exist, but
        /// the value list is empty or contains empty strings.</exception>
        public MimeMatcher(string nameSpace, string prefix)
        {
            this.mimes = Helper.PopulateFromConfiguration(nameSpace, prefix, MimesPropertyKey, true, false);
        }

        /// <summary>
        /// Implementation of the abstract method in base class, matches if the specified type matches one specified
        /// in the constructor, by case sensitive comparison.
        /// </summary>
        /// <param name="type">The file type.</param>
        /// <exception cref="FileMatchingException">Any error condition.</exception>
        /// <returns>True if the file is matched, false otherwise.</returns>
        protected override bool Match(FileType type)
        {
            return mimes.Contains(type.Mime);
        }
    }
}