/*
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDEVELOPER, TCSDESIGNER
 * Version: 1.0
 */

using System;
using System.Collections;
using TopCoder.Util.ConfigurationManager;
using System.IO;

namespace TopCoder.File.Statistics.Processor
{
    /// <summary>
    /// A base implementation that can be used for <c>IFileProcessor</c> instances. It provides alias and matcher
    /// support, including loading them from configuration file. It serves <c>TextLineCounter</c> and
    /// <c>CStyleLineCounter</c>. It can also be generically used by future implementations.
    /// </summary>
    /// <remarks>
    /// This class can load configuration from a namespace and a property name prefix. The following subproperties are
    /// supported:
    /// <list>
    /// <item>
    /// "alias", string value of the alias for this processor. This property is required and can not be empty.
    /// </item>
    /// <item>
    /// "matcher" which specifies a file matcher. It contains a "classname" which specifies the fully qualified class
    /// name of a <c>IFileMatcher</c> implementation that owns a constructor with a property argument.
    /// </item>
    /// </list>
    /// This class is immutable and thread-safe.
    /// </remarks>
    public abstract class FileProcessorBare : IFileProcessor
    {
        /// <summary>
        /// The property name suffix used to determine the alias for this file processor. This property is required.
        /// </summary>
        public const string AliasPropertyKey = ".alias";

        /// <summary>
        /// A property name suffix used to find the <c>IFileMatcher</c> for this file processor.
        /// </summary>
        public const string FileMatcherPropertyKey = ".matcher";

        /// <summary>
        /// A property name suffix to the matcher to find the specific (fully qualified) class name to instantiate
        /// through the reflection constructor contract.
        /// </summary>
        public const string ClassnamePropertyKey = ".classname";

        /// <summary>
        /// The alias used for this file processor. Only set in the constructor.
        /// </summary>
        private readonly string alias;

        /// <summary>
        /// This is the file matcher for the file processor (which determines if a file can/should be processed). It is
        /// set in constructor and never changed.
        /// </summary>
        private readonly IFileMatcher matcher;

        /// <summary>
        /// Exposes the file processor alias.
        /// </summary>
        /// <value>
        /// <c>alias</c>, a non-null, non-empty string.
        /// </value>
        public string Alias
        {
            get
            {
                return alias;
            }
        }

        /// <summary>
        /// Exposes the file matcher for the processor, which determines if a file can/should be processed.
        /// </summary>
        /// <value>
        /// <c>matcher</c>, a non-null <c>IFileMatcher</c> instance.
        /// </value>
        public IFileMatcher FileMatcher
        {
            get
            {
                return matcher;
            }
        }

        /// <summary>
        /// Exposes a list of all the supported metrics by this processor. This property is mostly for the end user.
        /// It does not fulfill any responsibility in the current framework.
        /// </summary>
        public abstract IList SupportedMetrics
        {
            get;
        }

        /// <summary>
        /// Constructor with both alias and matcher.
        /// </summary>
        /// <param name="alias">The alias for the file processor, non-empty, non-null.</param>
        /// <param name="matcher">The file matcher to use, non-null.</param>
        /// <exception cref="ArgumentNullException">If <c>alias</c> or <c>matcher</c> is null.</exception>
        /// <exception cref="ArgumentException">If <c>alias</c> is empty, when trimmed.</exception>
        protected FileProcessorBare(string alias, IFileMatcher matcher)
        {
            if (alias == null)
            {
                throw new ArgumentNullException("alias");
            }
            if (matcher == null)
            {
                throw new ArgumentNullException("matcher");
            }

            if (alias.Trim().Length == 0)
            {
                throw new ArgumentException("Empty string specified.", "alias");
            }

            this.alias = alias;
            this.matcher = matcher;
        }

        /// <summary>
        /// This constructor loads configuration from a namespace and a property name prefix. The non-empty alias and
        /// non-null file matcher are required. Any error will be wrapped in a <c>ConfigurationException</c>.
        /// </summary>
        /// <param name="nameSpace">The configuration manager namespace.</param>
        /// <param name="prefix">The prefix to the specific property names.</param>
        /// <exception cref="ArgumentNullException">If <c>nameSpace</c> or <c>prefix</c> is null.</exception>
        /// <exception cref="ArgumentException">If <c>nameSpace</c>, when trimmed, is empty.</exception>
        /// <exception cref="ConfigurationException">If any config error occurs.</exception>
        protected FileProcessorBare(string nameSpace, string prefix)
        {
            Helper.CheckConfigData(nameSpace, prefix);

            try
            {
                alias = ConfigManager.GetInstance().GetValue(nameSpace, prefix + AliasPropertyKey);
                if (alias == null)
                {
                    throw new ConfigurationException("No alias specified.");
                }
                if (alias.Trim().Length == 0)
                {
                    throw new ConfigurationException("Empty alias specified.");
                }

                string classname = ConfigManager.GetInstance().GetValue(nameSpace, prefix + FileMatcherPropertyKey +
                    ClassnamePropertyKey);

                if (classname == null)
                {
                    throw new ConfigurationException("No matcher class name specified.");
                }

                matcher = (IFileMatcher) Activator.CreateInstance(Type.GetType(classname), new object[] { nameSpace,
                    prefix + FileMatcherPropertyKey });
            }
            catch (ConfigurationException)
            {
                throw;
            }
            catch (Exception e)
            {
                throw new ConfigurationException("Configuration error.", e);
            }
        }

        /// <summary>
        /// Process the file and generate the result. A new result should be generated each time, with the metrics from
        /// <c>SupportedMetrics</c> used as keys.
        /// </summary>
        /// <param name="file">The file to process.</param>
        /// <exception cref="ArgumentNullException">If <c>file</c> is null.</exception>
        /// <exception cref="IOException">If any IO operation fails.</exception>
        /// <exception cref="FileProcessingException">If the processing fails.</exception>
        /// <returns>A stats result</returns>
        public abstract StatsResult Process(FileInfo file);
    }
}