/*
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDEVELOPER, TCSDESIGNER
 * Version: 2.0b
 */

using System;
using System.Collections;
using System.Collections.Specialized;
using System.Globalization;
using System.IO;
using TopCoder.Util.ConfigurationManager;

namespace TopCoder.File.Statistics
{
    /// <summary>
    /// A couple of static helper functions centered on the repeated need to read the configuration, validate it and
    /// get it into a case-insensitive hashtable for later matching.
    /// </summary>
    internal class Helper
    {                
        /// <summary>
        /// This dictionary maps the possible values for "true" for the "matchall" property. Any other value is
        /// considered false.
        /// </summary>
        private static readonly IDictionary TrueValues = Helper.CreateDictionary(3, false);
        
        /// <summary>
        /// Initializes the case-insensitive set of true values. We can't initialize an <c>IDictionary</c> in any more
        /// direct way.
        /// </summary>
        static Helper()
        {
            TrueValues.Add("true", null);
            TrueValues.Add("on", null);
            TrueValues.Add("yes", null);
        }
        
        /// <summary>
        /// Reads a value from the configuration and if it exists and matches one of the strings in TrueValues
        /// (by case-insensitive comparison), true is returned, otherwise false.
        /// </summary>
        /// <param name="nameSpace">The configuration manager namespace.</param>
        /// <param name="property">The configuration manager property key.</param>
        /// <returns><c>true</c> if one of the values for true has been specified, <c>false</c> otherwise.</returns>
        public static bool ReadBoolValue(string nameSpace, string property)
        {
            string value = ConfigManager.GetInstance().GetValue(nameSpace, property);

            if (value != null && TrueValues.Contains(value))
            {
                // If the configuration contains a recognized string for "true", according to a case-insensitive
                // comparison, set it accordingly.
                return true;
            }
            
            return false;
        }
        
        /// <summary>
        /// Takes a list of strings and puts them into a dictionary for efficient later comparison. Both case
        /// insensitive and sensitive comparison are supported. The list elements are also verified to be of correct
        /// type, not null, not empty.
        /// </summary>
        /// <param name="list">The list to populate the dictionary from.</param>
        /// <param name="paramName">The name of the original list param, used in exceptions (as this is an internal
        /// helper.</param>
        /// <param name="caseSensitive">Determines whether the resulting dictionary is case sensitive</param>
        /// <param name="allowEmpty">Allow empty list elements.</param>
        /// <exception cref="ArgumentNullException">If <c>list</c> is null.</exception>
        /// <exception cref="ArgumentException">If <c>list</c> is empty, or any element in <c>list</c> is null, not a
        /// string, or empty when trimmed.</exception>
        public static IDictionary PopulateFromList(IList list, string paramName, bool caseSensitive, bool allowEmpty)
        {
            if (list == null)
            {
                throw new ArgumentNullException(paramName);
            }

            if (list.Count == 0)
            {
                throw new ArgumentException("Empty list specified.", paramName);
            }

            IDictionary result = CreateDictionary(list.Count, caseSensitive);
            foreach (object value in list)
            {
                if (!(value is string))
                {
                    throw new ArgumentException("At least one element not a string, possibly null.", "types");
                }

                if (!allowEmpty && ((string) value).Trim().Length == 0)
                {
                    throw new ArgumentException("At least one string element empty when trimmed.", "types");
                }

                // The dictionary is only used as a set.
                result.Add(value, null);
            }

            return result;
        }

        /// <summary>
        /// Creates a new dictionaty, with the specified capacity. The comparison of strings can be made case
        /// insensitive. If so, the dictionary used bases the comparisons on the invariant culture.
        /// </summary>
        /// <param name="size">The intended size.</param>
        /// <param name="caseSensitive">Should string operations be case insensitive?</param>
        /// <returns>A dictionary with the requested capacity.</returns>
        public static IDictionary CreateDictionary(int size, bool caseSensitive)
        {
            // We encapsulates the needs here. For example, SortedList may turn out to be a wise choice as the
            // dictionaries are never modified. For now, HybridDictionary is used.
            return new HybridDictionary(size, !caseSensitive);
        }

        /// <summary>
        /// This method populates a case insensitive string dictionary from the specified namespace and configuration
        /// manager property (composed by prefix and suffix) in
        /// </summary>
        /// <param name="nameSpace">The namespace of the property.</param>
        /// <param name="prefix">The prefix to the property name.</param>
        /// <param name="suffix">The suffix to the property name (including dot).</param>
        /// <param name="caseSensitive">Should string operations be case insensitive?</param>
        /// <param name="allowEmpty">Allow empty list elements.</param>
        /// <exception cref="ArgumentNullException">If <c>nameSpace</c> or <c>prefix</c> is null (suffix not checked)
        /// </exception>
        /// <exception cref="ConfigurationException">If the namespace or property is not found, or if they exist, but
        /// the value list is empty or contains empty strings.</exception>
        /// <returns></returns>
        public static IDictionary PopulateFromConfiguration(string nameSpace, string prefix, string suffix,
            bool caseSensitive, bool allowEmpty)
        {
            string[] values = ReadValues(nameSpace, prefix, suffix, allowEmpty);

            IDictionary result = CreateDictionary(values.Length, caseSensitive);
            foreach (string str in values)
            {
                result.Add(str, null);
            }

            return result;
        }

        /// <summary>
        /// Reads all values from a specific property, including argument validation and string validation.
        /// </summary>
        /// <param name="nameSpace">The configuration manager namespace.</param>
        /// <param name="prefix">The prefix of the property name.</param>
        /// <param name="suffix">The suffix of the property name (including any '.' separator). No checks are performed
        /// on this parameter.</param>
        /// <param name="allowEmpty">Allow empty list elements.</param>
        /// <exception cref="ArgumentNullException">If <c>prefix</c> or <c>nameSpace</c> is null.</exception>
        /// <exception cref="ArgumentException">If <c>nameSpace</c>, when trimmed, is empty.
        /// </exception>
        /// <exception cref="ConfigurationException">If the namespace, property or its values are missing or any value
        /// is empty.</exception>
        public static string[] ReadValues(string nameSpace, string prefix, string suffix, bool allowEmpty)
        {
            Helper.CheckConfigData(nameSpace, prefix);

            string[] values = null;

            try
            {
                values = ConfigManager.GetInstance().GetValues(nameSpace, prefix + suffix);
            }
            catch (Exception e)
            {
                throw new ConfigurationException("Error in Configuration Manager.", e);
            }

            if (values == null)
            {
                throw new ConfigurationException("No value list present in configuration.");
            }
            if (values.Length == 0)
            {
                throw new ConfigurationException("Empty value list returned from configuration.");
            }

            if (!allowEmpty)
            {
                foreach (string str in values)
                {
                    if (str == null || str.Trim().Length == 0)
                    {
                        throw new ConfigurationException("Empty or null value returned from configuration.");
                    }
                }
            }

            return values;
        }

        /// <summary>
        /// Checks configuration parameters needed in configuration contract constructors.
        /// </summary>
        /// <param name="nameSpace">The configuration manager namespace.</param>
        /// <param name="prefix">The property name prefix for all strings.</param>
        /// <exception cref="ArgumentNullException">If <c>nameSpace</c> or <c>prefix</c> is null.</exception>
        /// <exception cref="ArgumentException">If <c>nameSpace</c>, when trimmed, is empty.</exception>
        /// <exception cref="ConfigurationException">If any config error occurs.</exception>
        public static void CheckConfigData(string nameSpace, string prefix)
        {
            if (nameSpace == null)
            {
                throw new ArgumentNullException("nameSpace");
            }
            if (prefix == null)
            {
                throw new ArgumentNullException("prefix");
            }

            if (nameSpace.Trim().Length == 0)
            {
                throw new ArgumentException("Empty namespace specified.", "nameSpace");
            }
            if (ConfigManager.GetInstance().GetNamespace(nameSpace) == null)
            {
                throw new ConfigurationException("Specified namespace not present.");
            }
        }
    }
}
