/*
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDEVELOPER, TCSDESIGNER
 * Version: 2.0b
 */

using System;
using System.Collections;
using System.IO;

namespace TopCoder.File.Statistics.Processor.LineCounter
{
    /// <summary>
    /// <c>CStyleLineCounter</c> counts lines for C/C++/C#/Java style source. It supports a single metric of
    /// "Line Count". It handles comments and string literals, but should only be considered an approximate count. It
    /// isn't resilient if exposed to non-syntactic code.
    /// </summary>
    /// <remarks>
    /// The configuration is equivalent to the base class, no differences (<see cref="TextLineCounter" />).
    /// The class is stateless and threadsafe. The component spec and the <c>CCountStateMachine</c> class documentation
    /// explains the counting method used.
    /// </remarks>
    public class CStyleLineCounter : FileProcessorBase, IArgumentHandler
    {
        /// <summary>
        /// This property key determines if @-style C# literals, possibly multiline, are to be supported, or not.
        /// </summary>
        public const string SupportAtLiteralPropertyKey = "support_at_literal";
        
        /// <summary>
        /// The name of the only supported metric.
        /// </summary>
        public const string Name = "Line Count";
        
        /// <summary>
        /// An array of the supported options, returned as a list in <c>ProposeOptions</c>.
        /// </summary>
        private ArgumentOption[] options = new ArgumentOption[]
            { new ArgumentOption(SupportAtLiteralPropertyKey,
                                 "Handle escaping and the multiline support in C# '@' literals.",
                                 false) };

        /// <summary>
        /// The display name (XML element name) of the only supported metric.
        /// </summary>
        public const string DisplayName = "line";
        
        /// <summary>
        /// This setting controls whether "at" (char 64) C# literals are handled properly, regarding multilineness
        /// and DQ escaping.
        /// </summary>
        private bool supportAtLiteral = false;

        /// <summary>
        /// Description of the only supported metric.
        /// </summary>
        private const string Description = "Line count (in a C style source file)";

        /// <summary>
        /// An immutable reference to the only supported metric.
        /// </summary>
        private static readonly Metric LineMetric = new Metric(Name, DisplayName, Description);

        /// <summary>
        /// Exposes a list of the supported metrics, in this case only one, line count.
        /// </summary>
        /// <value>
        /// A non-null <c>IList</c> with exactly one <c>Metric</c> element.
        /// </value>
        public override IList SupportedMetrics
        {
            get
            {
                IList list = new ArrayList();
                list.Add(LineMetric);

                return list;
            }
        }

        /// <summary>
        /// Constructor with both alias and matcher.
        /// </summary>
        /// <param name="alias">The alias for the file processor, non-empty, non-null.</param>
        /// <param name="matcher">The file matcher to use, non-null.</param>
        /// <exception cref="ArgumentNullException">If <c>alias</c> or <c>matcher</c> is null.</exception>
        /// <exception cref="ArgumentException">If <c>alias</c> is empty, when trimmed.</exception>
        public CStyleLineCounter(string alias, IFileMatcher matcher)
            : base(alias, matcher)
        {
        }
        
        /// <summary>
        /// Constructor alias, matcher and a setting whether to handle C# "@" literals.
        /// </summary>
        /// <param name="alias">The alias for the file processor, non-empty, non-null.</param>
        /// <param name="matcher">The file matcher to use, non-null.</param>
        /// <param name="supportAtLiteral">Should C#-style @"" literals be handled?</param>
        /// <exception cref="ArgumentNullException">If <c>alias</c> or <c>matcher</c> is null.</exception>
        /// <exception cref="ArgumentException">If <c>alias</c> is empty, when trimmed.</exception>
        public CStyleLineCounter(string alias, IFileMatcher matcher, bool supportAtLiteral)
            : this(alias, matcher)
        {
            this.supportAtLiteral = supportAtLiteral;
        }

        /// <summary>
        /// This constructor loads configuration from a namespace and a property name prefix. The non-empty alias and
        /// non-null file matcher are required. Any error will be wrapped in a <c>ConfigurationException</c>.
        /// </summary>
        /// <param name="nameSpace">The configuration manager namespace.</param>
        /// <param name="prefix">The prefix to the specific property names.</param>
        /// <exception cref="ArgumentNullException">If <c>nameSpace</c> or <c>prefix</c> is null.</exception>
        /// <exception cref="ConfigurationException">Any configuration error.</exception>
        public CStyleLineCounter(string nameSpace, string prefix)
            : base(nameSpace, prefix)
        {
            supportAtLiteral = Helper.ReadBoolValue(nameSpace, prefix + SupportAtLiteralPropertyKey);
        }

        /// <summary>
        /// Opens the file, counts its lines by the means of a state machine and returns the result.
        /// </summary>
        /// <param name="file">The file to process.</param>
        /// <exception cref="ArgumentNullException">If <c>file</c> is null.</exception>
        /// <exception cref="IOException">For any IO error.</exception>
        /// <returns>A non-null <c>StatsResult</c> with a value for the supported metric.</returns>
        public override StatsResult Process(FileInfo file)
        {
            SimpleStateMachine machine = new CCountStateMachine(supportAtLiteral);
            machine.Process(file);

            StatsResult result = new StatsResult();
            result[LineMetric] = machine.Lines;

            return result;
        }

        /// <summary>
        /// Method defined in the <c>IArgumentHandler</c> interface, to specify a list of options.
        /// </summary>
        /// <returns>
        /// An <c>IList</c> with one <c>ArgumentOption</c> instance specifying the "support_at_literal" option.
        /// </returns>
        public IList ProposeOptions()
        {
            return options;
        }

        /// <summary>
        /// Argument handling method defined in the <c>IArgumentHandler</c> interface. Actually implements the settings
        /// specified by the arguments.
        /// </summary>
        /// <remarks>
        /// <c>supportAtLiteral</c> is set to <c>true</c> if and only if the option "support_at_literal" has been
        /// specified. The actual argument string is ignored.
        /// </remarks>
        /// <param name="arguments">A map of those switches (keys, strings) that were reported as supported in
        /// <c>ProposeOptions</c> and their argument values (as map values).</param>
        /// <exception cref="ArgumentNullException">If <c>arguments</c> is null.</exception>
        /// <exception cref="ArgumentException">If the "support_at_literal" key exists, but the value is of non-string
        /// type (including null).</exception>
        public void HandleArguments(IDictionary arguments)
        {
            if (arguments == null)
            {
                throw new ArgumentNullException("arguments");
            }
            if (arguments.Contains(SupportAtLiteralPropertyKey))
            {
                supportAtLiteral = true;
                if (!(arguments[SupportAtLiteralPropertyKey] is string))
                {
                    throw new ArgumentException("Argument specified with argument value of non-string type.",
                                                "arguments");
                }
            }
        }

        /// <summary>
        /// A relatively simple state machine defining a new line as '{', ';', handling comments and string literals,
        /// including @-escaped C# literals.
        /// <see cref="SimpleStateMachine" />
        /// </summary>
        private class CCountStateMachine : SimpleStateMachine
        {
            /// <summary>
            /// A precached delegate to the matching state method.
            /// </summary>
            private readonly StateDelegate stateSlash;
            
            /// <summary>
            /// A precached delegate to the matching state method.
            /// </summary>
            private readonly StateDelegate stateStar;
            
            /// <summary>
            /// A precached delegate to the matching state method.
            /// </summary>
            private readonly StateDelegate stateSQBackslash;
            
            /// <summary>
            /// A precached delegate to the matching state method.
            /// </summary>
            private readonly StateDelegate stateDQBackslash;
            
            /// <summary>
            /// A precached delegate to the matching state method.
            /// </summary>
            private readonly StateDelegate stateSingleQuote;
            
            /// <summary>
            /// A precached delegate to the matching state method.
            /// </summary>
            private readonly StateDelegate stateDoubleQuote;
            
            /// <summary>
            /// A precached delegate to the matching state method.
            /// </summary>
            private readonly StateDelegate stateBlockComment;
            
            /// <summary>
            /// A precached delegate to the matching state method.
            /// </summary>
            private readonly StateDelegate stateLineComment;
            
            /// <summary>
            /// A precached delegate to the matching state method.
            /// </summary>
            private readonly StateDelegate stateStart;
            
            /// <summary>
            /// A precached delegate to the matching state method OR <c>StateStart</c>, if @ literal handling has been
            /// disabled.
            /// </summary>
            private readonly StateDelegate stateChar64;
            
            /// <summary>
            /// A precached delegate to the matching state method.
            /// </summary>
            private readonly StateDelegate stateChar64Literal;
            
            /// <summary>
            /// A precached delegate to the matching state method.
            /// </summary>
            private readonly StateDelegate stateChar64DQ;
            
            /// <summary>
            /// Prepares the state machine for counting with or without C#-style "@" literals.
            /// 
            /// This is done by controlling the target of the stateChar64 delegate, so it sometimes points to the
            /// StateStart method, thereby disabling the complete @ character handling.
            /// </summary>           
            public CCountStateMachine(bool supportAtLiterals)
            {
                stateSlash = new StateDelegate(StateSlash);
                stateStar = new StateDelegate(StateStar);
                stateSQBackslash = new StateDelegate(StateSQBackslash);
                stateDQBackslash = new StateDelegate(StateDQBackslash);
                stateSingleQuote = new StateDelegate(StateSingleQuote);
                stateDoubleQuote = new StateDelegate(StateDoubleQuote);
                stateBlockComment = new StateDelegate(StateBlockComment);
                stateLineComment = new StateDelegate(StateLineComment);
                stateStart = new StateDelegate(StateStart);
                
                // Note this! We check the setting and handle it accordingly.
                stateChar64 = supportAtLiterals ? new StateDelegate(StateChar64) : stateStart;
                
                // This is never entered when it's turned off.
                stateChar64Literal = new StateDelegate(StateChar64Literal);
                stateChar64DQ = new StateDelegate(StateChar64DQ);
            }
            
            /// <summary>
            /// A state method for handling a "free" slash in normal code. This means that a line comment or block
            /// comment (/* and //, respectively), might start.
            /// </summary>
            /// <param name="nextChar">The next character, as an int, where -1 is EOF.</param>
            /// <returns>The delegate for the state to use for the next character.</returns>
            public StateDelegate StateSlash(int nextChar)
            {
                switch (nextChar)
                {
                    case '*': // /*
                        return stateBlockComment;
                    case '/': // //
                        return stateLineComment;
                    default: // normal code
                        return StateStart(nextChar);
                }
            }

            /// <summary>
            /// A state method for handling an asterisk within a block comment. After that, a slash might follow, to
            /// end the comment.
            /// </summary>
            /// <param name="nextChar">The next character, as an int, where -1 is EOF.</param>
            /// <returns>The delegate for the state to use for the next character.</returns>
            public StateDelegate StateStar(int nextChar)
            {
                switch (nextChar)
                {
                    case '/': // */, comment ended, normal code again
                        return stateStart;
                    default: // we're within a block comment, nothing special should be needed, but let the general
                        // code look at it
                        return StateBlockComment(nextChar);
                }
            }

            /// <summary>
            /// A state method for handling a backslash within a character literal. The next character is "swallowed"
            /// and we go back to a normal single quote literal.
            /// </summary>
            /// <param name="nextChar">The next character, as an int, where -1 is EOF.</param>
            /// <returns>The delegate for the state to use for the next character.</returns>
            public StateDelegate StateSQBackslash(int nextChar)
            {
                switch (nextChar)
                {
                    case '\n':
                    case '\r':
                        return stateStart; // Even with escaping, these are an end
                    default: // Get back to the single quote state, ignoring this specific character
                        return stateSingleQuote; // NOT (nextChar), we ignore one character
                }
            }

            /// <summary>
            /// A state method for handling a backslash within a normal string literal. The next character is
            /// "swallowed" and we go back to a normal double quote literal.
            /// </summary>
            /// <param name="nextChar">The next character, as an int, where -1 is EOF.</param>
            /// <returns>The delegate for the state to use for the next character.</returns>
            public StateDelegate StateDQBackslash(int nextChar)
            {
                switch (nextChar)
                {
                    case '\n':
                    case '\r':
                        return stateStart; // Even with escaping, these are an end
                    default: // Get back to the double quote state, ignoring this specific character
                        return stateDoubleQuote; // NOT (nextChar), we ignore one character
                }
            }

            /// <summary>
            /// A state method for handling a character literal. The only special characters are the ending single
            /// quote and backslash for escaping. New lines are considered to end the state, no matter what.
            /// </summary>
            /// <param name="nextChar">The next character, as an int, where -1 is EOF.</param>
            /// <returns>The delegate for the state to use for the next character.</returns>
            public StateDelegate StateSingleQuote(int nextChar)
            {
                switch (nextChar)
                {
                    case '\n':
                    case '\r':
                    case '\'':
                        return stateStart; // These are an end
                    case '\\': // Escaping, most importantly the case of \'
                        return stateSQBackslash;
                    default: // Go on
                        return stateSingleQuote;
                }
            }

            /// <summary>
            /// A state method for handling a string literal. The only special characters are the ending double quote
            /// and backslash for escaping. New lines are considered to end the state, no matter what.
            /// </summary>
            /// <param name="nextChar">The next character, as an int, where -1 is EOF.</param>
            /// <returns>The delegate for the state to use for the next character.</returns>
            public StateDelegate StateDoubleQuote(int nextChar)
            {
                switch (nextChar)
                {
                    case '\n':
                    case '\r':
                    case '"':
                        return stateStart; // These are an end
                    case '\\': // Escaping, most importantly the case of \"
                        return stateDQBackslash;
                    default: // Go on
                        return stateDoubleQuote;
                }
            }

            /// <summary>
            /// A state method for handling a normal character within a block comment. We can reach an asterisk.
            /// </summary>
            /// <param name="nextChar">The next character, as an int, where -1 is EOF.</param>
            /// <returns>The delegate for the state to use for the next character.</returns>
            public StateDelegate StateBlockComment(int nextChar)
            {
                switch (nextChar)
                {
                    case '*':
                        return stateStar;
                    default: // Go on
                        return stateBlockComment;
                }
            }

            /// <summary>
            /// A state method for handling line comments. They can only be ended by a new line.
            /// </summary>
            /// <param name="nextChar">The next character, as an int, where -1 is EOF.</param>
            /// <returns>The delegate for the state to use for the next character.</returns>
            public StateDelegate StateLineComment(int nextChar)
            {
                switch (nextChar)
                {
                    case '\n':
                    case '\r':
                        return stateStart; // These are an end
                    default:
                        return stateLineComment;
                }
            }



            /// <summary>
            /// This is the starting state and the state of normal code. We define a line start as a { or ;. This is
            /// not very accurate. A for statement is generally considered to be one line, but it will easily be
            /// several with this interpretation. Also, inline array definitions with { } may disturb this.
            ///
            /// This method will find starts of comments and character/string literals.
            /// </summary>
            /// <param name="nextChar">The next character, as an int, where -1 is EOF.</param>
            /// <returns>The delegate for the state to use for the next character.</returns>
            public override StateDelegate StateStart(int nextChar)
            {
                switch (nextChar)
                {
                    case ';':
                    case '{': // These are the only cases where the line count is increased
                        lines++;
                        return stateStart;

                    case '/': // Possible start of comment, block or line
                        return stateSlash;

                    case '\'': // Character literal start
                        return stateSingleQuote;

                    case '"': // String literal start
                        return stateDoubleQuote;

                    case '@': // This may be the start of a @-escaped C# string literal
                        // Note that stateChar64 MIGHT be a StateStart delegate, depending on our
                        // @ handling setting.
                        return stateChar64;

                    default:
                        return stateStart;
                }
            }

            /// <summary>
            /// This is a state handler method intended to handle @ characters, which may be the start of @ escaped
            /// string literals in C#.
            /// </summary>
            /// <param name="nextChar">The next character, as an int, where -1 is EOF.</param>
            /// <returns>The delegate for the state to use for the next character.</returns>
            public StateDelegate StateChar64(int nextChar)
            {
                switch (nextChar)
                {
                    case '"':
                        return stateChar64Literal;

                    default: // Not a literal start, get back
                        return StateStart(nextChar);
                }
            }

            /// <summary>
            /// This is a state handler method intended to handle @ escaped string literals. They can be ended only
            /// by a double quote (not even a new line), but only if it's not a quadruple quote (""), which is just
            /// an escaped double quote inside the string.
            /// </summary>
            /// <param name="nextChar">The next character, as an int, where -1 is EOF.</param>
            /// <returns>The delegate for the state to use for the next character.</returns>
            public StateDelegate StateChar64Literal(int nextChar)
            {
                switch (nextChar)
                {
                    case '"':
                        return stateChar64DQ;

                    default: // Go on
                        return stateChar64Literal;
                }
            }

            /// <summary>
            /// This is a state handler method intended to handle @ escaped string literal endings, and escaped
            /// double quotes inside the string.
            /// </summary>
            /// <param name="nextChar">The next character, as an int, where -1 is EOF.</param>
            /// <returns>The delegate for the state to use for the next character.</returns>
            public StateDelegate StateChar64DQ(int nextChar)
            {
                switch (nextChar)
                {
                    case '"': // This was an escaped DQ, get back
                        return stateChar64Literal;

                    default: // The next character is normal code, handle it as such
                        return StateStart(nextChar);
                }
            }
        }
    }
}
