/*
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDEVELOPER, TCSDESIGNER
 * Version: 2.0b
 */

using System;
using System.Collections;
using System.IO;

namespace TopCoder.File.Statistics.Processor.LineCounter
{
    /// <summary>
    /// <c>TextLineCounter</c> counts lines for text files. It supports a single metric of "Line Count".
    /// </summary>
    /// <remarks>
    /// The configuration is equivalent to the base class, no differences (<see cref="TextLineCounter" />).
    /// The class is stateless and threadsafe.
    /// </remarks>
    public class TextLineCounter : FileProcessorBase, IArgumentHandler
    {
        /// <summary>
        /// A configuration manager property key. If it exists and has the value "true" / "yes" / "on" (according to
        /// case insensitive comparison), blank lines will be ignored during counting.
        /// </summary>
        public const string IgnoreBlankLinePropertyKey = "ignore_blank_line";
        
        /// <summary>
        /// An array of the supported options, returned as a list in <c>ProposeOptions</c>.
        /// </summary>
        private static readonly ArgumentOption[] options = new ArgumentOption[]
            { new ArgumentOption(IgnoreBlankLinePropertyKey,
                                 "Count only non-blank lines.",
                                 false) };
        
        /// <summary>
        /// The name of the only supported metric.
        /// </summary>
        public const string Name = "Line Count";

        /// <summary>
        /// The display name (XML element name) of the only supported metric.
        /// </summary>
        public const string DisplayName = "line";

        /// <summary>
        /// Description of the only supported metric.
        /// </summary>
        private const string Description = "Line count (in a text file)";

        /// <summary>
        /// An immutable reference to the only supported metric.
        /// </summary>
        private static readonly Metric LineMetric = new Metric(Name, DisplayName, Description);
        
        /// <summary>
        /// A setting for whether to ignore blank lines. Note that this is distinct from the CRLF handling, the results
        /// when "not" ignoring blank lines will still ignore consecutive CRLF / LFCR pairs (but not CRLFCRLF).
        /// </summary>
        private bool ignoreBlankLine = false;

        /// <summary>
        /// Exposes a list of the supported metrics, in this case only one, line count.
        /// </summary>
        /// <value>
        /// A non-null <c>IList</c> with exactly one <c>Metric</c> element.
        /// </value>
        public override IList SupportedMetrics
        {
            get
            {
                IList list = new ArrayList();
                list.Add(LineMetric);

                return list;
            }
        }

        /// <summary>
        /// Constructor with both alias and matcher.
        /// </summary>
        /// <param name="alias">The alias for the file processor, non-empty, non-null.</param>
        /// <param name="matcher">The file matcher to use, non-null.</param>
        /// <exception cref="ArgumentNullException">If <c>alias</c> or <c>matcher</c> is null.</exception>
        /// <exception cref="ArgumentException">If <c>alias</c> is empty, when trimmed.</exception>
        public TextLineCounter(string alias, IFileMatcher matcher)
            : base(alias, matcher)
        {
        }
        
        /// <summary>
        /// Constructor with both alias and matcher.
        /// </summary>
        /// <param name="alias">The alias for the file processor, non-empty, non-null.</param>
        /// <param name="matcher">The file matcher to use, non-null.</param>
        /// <param name="ignoreBlankLine">Determines whether blank lines should be counted or not.</param>
        /// <exception cref="ArgumentNullException">If <c>alias</c> or <c>matcher</c> is null.</exception>
        /// <exception cref="ArgumentException">If <c>alias</c> is empty, when trimmed.</exception>
        public TextLineCounter(string alias, IFileMatcher matcher, bool ignoreBlankLine)
            : this(alias, matcher)
        {
            this.ignoreBlankLine = ignoreBlankLine;
        }

        /// <summary>
        /// This constructor loads configuration from a namespace and a property name prefix. The non-empty alias and
        /// non-null file matcher are required. Any error will be wrapped in a <c>ConfigurationException</c>.
        /// </summary>
        /// <param name="nameSpace">The configuration manager namespace.</param>
        /// <param name="prefix">The prefix to the specific property names.</param>
        /// <exception cref="ArgumentNullException">If <c>nameSpace</c> or <c>prefix</c> is null.</exception>
        /// <exception cref="ConfigurationException">Any configuration error.</exception>
        public TextLineCounter(string nameSpace, string prefix)
            : base(nameSpace, prefix)
        {
            ignoreBlankLine = Helper.ReadBoolValue(nameSpace, prefix + IgnoreBlankLinePropertyKey);
        }

        /// <summary>
        /// Opens the file, counts its lines by the means of a state machine and returns the result.
        /// </summary>
        /// <param name="file">The file to process.</param>
        /// <exception cref="ArgumentNullException">If <c>file</c> is null.</exception>
        /// <exception cref="IOException">For any IO error.</exception>
        /// <returns>A non-null <c>StatsResult</c> with a value for the supported metric.</returns>
        public override StatsResult Process(FileInfo file)
        {
            SimpleStateMachine machine = new TextCountStateMachine(ignoreBlankLine);
            machine.Process(file);

            StatsResult result = new StatsResult();
            result[LineMetric] = machine.Lines;

            return result;
        }
        
        /// <summary>
        /// Method defined in the <c>IArgumentHandler</c> interface, to specify a list of options.
        /// </summary>
        /// <returns>
        /// An <c>IList</c> with one <c>ArgumentOption</c> instance specifying the "support_at_literal" option.
        /// </returns>
        public IList ProposeOptions()
        {
            return options;
        }

        /// <summary>
        /// Argument handling method defined in the <c>IArgumentHandler</c> interface. Actually implements the settings
        /// specified by the arguments.
        /// </summary>
        /// <remarks>
        /// <c>ignoreBlankLine</c> is set to <c>true</c> if and only if the option "ignore_blank_line" has been
        /// specified. The actual argument string is ignored.
        /// </remarks>
        /// <param name="arguments">A map of those switches (keys, strings) that were reported as supported in
        /// <c>ProposeOptions</c> and their argument values (as map values).</param>
        /// <exception cref="ArgumentNullException">If <c>arguments</c> is null.</exception>
        /// <exception cref="ArgumentException">If the "ignore_blank_line" key exists, but the value is of non-string
        /// type (including null).</exception>
        public void HandleArguments(IDictionary arguments)
        {
            if (arguments == null)
            {
                throw new ArgumentNullException("arguments");
            }
            if (arguments.Contains(IgnoreBlankLinePropertyKey))
            {
                ignoreBlankLine = true;
                if (!(arguments[IgnoreBlankLinePropertyKey] is string))
                {
                    throw new ArgumentException("Argument specified with argument value of non-string type.",
                                                "arguments");
                }
            }
        }

        /// <summary>
        /// A simple state machine collapsing all CRLF and LFCR to single lines when counted.
        /// <see cref="SimpleStateMachine" />
        /// </summary>
        private class TextCountStateMachine : SimpleStateMachine
        {
            /// <summary>
            /// A local copy of the setting used in <c>StateStart</c> to determine when to count.
            /// </summary>
            private bool ignoreBlankLine;
            
            /// <summary>
            /// A precached delegate to the corresponding method. Set in constructor.
            /// </summary>
            private readonly StateDelegate stateCR;
            
            /// <summary>
            /// A precached delegate to the corresponding method. Set in constructor.
            /// </summary>
            private readonly StateDelegate stateLF;
            
            /// <summary>
            /// A precached delegate to the corresponding method. Set in constructor.
            /// </summary>
            private readonly StateDelegate stateStart;
            
            /// <summary>
            /// A precached delegate to the corresponding method. Set in constructor.
            /// </summary>
            private readonly StateDelegate stateInLine;
            
            /// <summary>
            /// Constructor precaching delegates and setting the internal setting according to the argument.
            /// </summary>
            /// <param name="ignoreBlankLine">Ignore blank lines.</param>
            public TextCountStateMachine(bool ignoreBlankLine)
            {
                this.ignoreBlankLine = ignoreBlankLine;
                
                stateCR = new StateDelegate(StateCR);
                stateLF = new StateDelegate(StateLF);
                stateStart = new StateDelegate(StateStart);
                stateInLine = new StateDelegate(StateInLine);
            }
            
            /// <summary>
            /// A state method for handling carriage return.
            /// </summary>
            /// <param name="nextChar">The next character, as an int, where -1 is EOF.</param>
            /// <returns>The delegate for the state to use for the next character.</returns>
            public StateDelegate StateCR(int nextChar)
            {
                switch (nextChar)
                {
                    // The only difference is that CRLF is considered to be ONE newline, so we jump directly to the
                    // next stateStart, but NOT calling StateStart itself on this character.
                    case '\n':
                        return stateStart;
                    default:
                        return StateStart(nextChar);
                }
            }

            /// <summary>
            /// A state method for handling line feed.
            /// </summary>
            /// <param name="nextChar">The next character, as an int, where -1 is EOF.</param>
            /// <returns>The delegate for the state to use for the next character.</returns>
            public StateDelegate StateLF(int nextChar)
            {
                switch (nextChar)
                {
                    // The only difference is that LFCR is considered to be ONE newline, so we jump directly to the
                    // next stateStart, but NOT calling StateStart itself on this character.
                    case '\r':
                        return stateStart;
                    default:
                        return StateStart(nextChar);
                }
            }

            /// <summary>
            /// A state method for handling the normal character flow. CR, LF and EOF trigger line number increments.
            /// CR and LF affects the state for the next character to handle CRLF/LFCR.
            /// </summary>
            /// <param name="nextChar">The next character, as an int, where -1 is EOF.</param>
            /// <returns>The delegate for the state to use for the next character.</returns>
            public override StateDelegate StateStart(int nextChar)
            {
                switch (nextChar)
                {
                    case '\r':
                        if (!ignoreBlankLine)
                        {
                            lines++;
                        }
                        return stateCR;
                    case '\n':
                        if (!ignoreBlankLine)
                        {
                            lines++;
                        }
                        return stateLF;
                    case -1: // EOF
                        if (!ignoreBlankLine)
                        {
                            lines++; // No state change needed, this is the end
                        }
                        return stateStart;
                    default: // Just go on to the "within line" delegate.
                        return stateInLine;
                }
            }
            
            /// <summary>
            /// A state method for handling the normal character flow. CR, LF and EOF trigger line number increments.
            /// CR and LF affects the state for the next character to handle CRLF/LFCR. This method is used for every
            /// non-first character of a line, to handle counting of only non-blank lines.
            /// </summary>
            /// <param name="nextChar">The next character, as an int, where -1 is EOF.</param>
            /// <returns>The delegate for the state to use for the next character.</returns>
            public StateDelegate StateInLine(int nextChar)
            {
                switch (nextChar)
                {
                    case '\r':
                        lines++;
                        return stateCR;
                    case '\n':
                        lines++;
                        return stateLF;
                    case -1: // EOF
                        lines++; // No state change needed, this is the end
                        return stateStart; // The return value is really irrelevant...
                    default: // Just go on...
                        return stateInLine;
                }
            }
        }
    }
}
