/*
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDEVELOPER, TCSDESIGNER
 * Version: 2.0b
 */

using System;
using System.IO;
using System.Collections;

namespace TopCoder.File.Statistics.Reporting
{
    /// <summary>
    /// This class provides a <c>IStatsReporting</c> implementation base. It aggregates results from each processor and
    /// classifies them based on single file, directory or alias. It also provides an overall stats. It defers the
    /// actual rendering logic to concrete implementations (<c>BasicStatsReporting</c> and <c>XmlStatsReporting</c>).
    /// </summary>
    /// <remarks>
    /// This class has a constructor with a namespace and a property for extension purposes. There is no real
    /// configuration parsing logic there at this point in time.
    /// </remarks>
    public abstract class AbstractStatsReporting : IStatsReporting
    {
        /// <summary>
        /// The stats mapped to each file. The file path is relative to base dir or absolute if no base dir. Its
        /// reference never changes.
        /// </summary>
        private readonly StatsCollection fileStats = new StatsCollection();

        /// <summary>
        /// The stats mapped to each directory. The dir path is relative to base dir or absolute if no base dir. Its
        /// reference never changes.
        /// </summary>
        private readonly StatsCollection directoryStats = new StatsCollection();

        /// <summary>
        /// The stats mapped to each (processor) alias.
        /// </summary>
        private readonly StatsCollection aliasStats = new StatsCollection();

        /// <summary>
        /// The overall stats (since the last call to <c>Reset</c>).
        /// </summary>
        private StatsResult overallStats = new StatsResult();

        /// <summary>
        /// The base dir for the reporting. This variable is nullable (meaning all paths are absolute).
        /// </summary>
        private DirectoryInfo baseDirectory = null;
        
        /// <summary>
        /// A cached string representation of the value of <c>baseDirectory</c>.
        /// </summary>
        private string baseDirFull = null;
        
        /// <summary>
        /// A cache of canonical file/directory names to avoid real Win32 API calls.
        /// </summary>
        private IDictionary cachedCanonicals = new Hashtable();

        /// <summary>
        /// Getter exposing the file stats. Note that this is not a copy.
        /// </summary>
        /// <value>
        /// <c>fileStats</c>, a <c>StatsCollection</c> for all files, all processors aggregated.
        /// </value>
        public StatsCollection FileStats
        {
            get
            {
                return fileStats;
            }
        }

        /// <summary>
        /// Getter exposing the directory stats. Note that this is not a copy.
        /// </summary>
        /// <value>
        /// <c>directoryStats</c>, a <c>StatsCollection</c> for all directories with aggregated data.
        /// </value>
        public StatsCollection DirectoryStats
        {
            get
            {
                return directoryStats;
            }
        }

        /// <summary>
        /// Getter exposing the file processor alias stats. Note that this is not a copy.
        /// </summary>
        /// <value>
        /// <c>aliasStats</c>, a <c>StatsCollection</c> for every file processor alias, aggregated over all files.
        /// </value>
        public StatsCollection AliasStats
        {
            get
            {
                return aliasStats;
            }
        }

        /// <summary>
        /// Getter exposing the overall stats, as a clone.
        /// </summary>
        /// <value>
        /// A clone of the private <c>overallStats</c> member.
        /// </value>
        public StatsResult OverallStats
        {
            get
            {
                return new StatsResult(overallStats);
            }
        }

        /// <summary>
        /// Exposes the base directory variable.
        /// </summary>
        /// <value>
        /// A <c>DirectoryInfo</c>, possibly null.
        /// </value>
        public DirectoryInfo BaseDirectory
        {
            get
            {
                return baseDirectory;
            }
        }

        /// <summary>
        /// Default constructor. Does nothing.
        /// </summary>
        protected AbstractStatsReporting()
        {
        }

        /// <summary>
        /// Constructor taking a reference to a configuration manager property. Mainly provided for future
        /// extensibility. In this release, the parameters are only checked superficially.
        /// </summary>
        /// <param name="nameSpace">The configuration manager namespace.</param>
        /// <param name="prefix">The prefix to the properties for this reporting method.</param>
        /// <exception cref="ArgumentNullException">If <c>nameSpace</c> or <c>prefix</c> is null.</exception>
        /// <exception cref="ArgumentException">If <c>nameSpace</c>, when trimmed, is empty.</exception>
        /// <exception cref="ConfigurationException">If any config error occurs.</exception>
        protected AbstractStatsReporting(string nameSpace, string prefix)
        {
            Helper.CheckConfigData(nameSpace, prefix);
        }

        /// <summary>
        /// A wrapper around the <c>Path</c> static methods <c>GetDirectoryName</c> and
        /// <c>GetPathRoot</c>, unifying the behavior.
        /// </summary>
        /// <param name="path">The path to find the parent within.</param>
        /// <returns>The parent directory, or null if none.</returns>
        private static string GetDirectoryName(string path)
        {
            string parent = Path.GetDirectoryName(path);
            if (parent == null)
            {
                parent = Path.GetPathRoot(path);
                if (parent == "")
                {
                    parent = null; // Nothing added, stop!
                }
            }
            
            return parent;
        }
        
        /// <summary>
        /// A wrapper around the <c>Path</c> static method <c>GetFileName</c> that returns the path back
        /// unchanged if that method would return null or empty.
        /// </summary>
        /// <param name="path">The path to find the parent within.</param>
        /// <returns>The parent directory, or null if none.</returns>
        private static string GetFileName(string path)
        {
            string newPath = Path.GetFileName(path);
            if (path == "" || path == null)
            {
                newPath = path;
            }
            
            return newPath;
        }

        /// <summary>
        /// Gets a canonical file name by doing a file system query (search) for the name. That can sometimes resolve
        /// short/long names and other effects in a predictable manner.
        /// </summary>
        /// <param name="directory">The directory with the item.</param>
        /// <param name="pattern">The name of the item.</param>
        /// <returns></returns>
        private string GetCanonical(string directory, string pattern)
        {
            // If we can't do a query, don't!
            if (directory == null)
            {
                return pattern;
            }            
            
            if (pattern == null || pattern == "")
            {
                return GetFileName(directory);
            }
            
            string key = directory + "\\" + pattern;
            
            if (cachedCanonicals.Contains(key))
            {
                return (string) cachedCanonicals[key];
            }
            
            string[] entries = Directory.GetFileSystemEntries(directory, pattern);

            string val;
            if (entries.Length == 0)
            {
                val = pattern;
            }
            else
            {
                val = Path.GetFileName(entries[0]);
            }
            
            cachedCanonicals[key] = val;            
            return val;
        }

        /// <summary>
        /// Aggregate a result into the reporting.
        /// </summary>
        /// <remarks>
        /// This means that the information is recorded in the overall stats, file name based stats, and for every
        /// directory "upwards" in the path, up to, but not including, any specified base directory. This is because
        /// the base directory stats would be positively equivalent to the overall stats.
        /// </remarks>
        /// <param name="processor">The processor that generated the result.</param>
        /// <param name="file">The file that was processed.</param>
        /// <param name="result">The result to aggregate.</param>
        /// <exception cref="ArgumentNullException">If any parameter is null.</exception>
        /// <exception cref="StatsReportingException">If the file is not present in the directory tree to the
        /// specified base dir.</exception>
        public void Aggregate(IFileProcessor processor, FileInfo file, StatsResult result)
        {
            if (processor == null)
            {
                throw new ArgumentNullException("processor");
            }
            if (file == null)
            {
                throw new ArgumentNullException("file");
            }
            if (result == null)
            {
                throw new ArgumentNullException("result");
            }

            overallStats.Aggregate(result);
            aliasStats.Add(processor.Alias, result);

            ArrayList list = new ArrayList();
            file = new FileInfo(file.FullName);
            if (!file.Exists)
            {
                throw new StatsReportingException("Th file specified should exist.");
            }
            
            string dir = GetDirectoryName(file.FullName);

            list.Add(GetCanonical(dir, file.Name));

            // Walk up the directory tree until we reach the root or the base directory
            while (dir != null && dir != baseDirFull)
            {
                string parent = GetDirectoryName(dir);
                if (parent == dir)
                {
                    list.Add(parent);
                    break;                    
                }
                
                list.Add(GetCanonical(parent, GetFileName(dir)));
                dir = parent;
            }

            // If there is a base directory, but we reached the root, something is wrong
            if (baseDirectory != null && dir == null)
            {
                throw new StatsReportingException(string.Format("File {0} not within base directory {1}.", file,
                    baseDirectory));
            }

            // A StringBuffer would require some work duplicating the special root path handling, and it still
            // wouldn't help much as we need to churn out a string at each step.
            string path = "";
            // Reconstruct a canonical path, add for each directory
            for (int i = list.Count - 1; i > 0; i--)
            {
                path = Path.Combine(path, (string) list[i]);
                
                directoryStats.Add(path, result);
            }

            // And now for the file name.
            path = Path.Combine(path, (string) list[0]);
            fileStats.Add(path, result);
        }

        /// <summary>
        /// Reset the reporting. The base directory can be null, to indicate that absolute paths are desired.
        /// </summary>
        /// <param name="baseDirectory">The base directory to use, may even be null.</param>
        /// <exception cref="ArgumentException">If a non-existing base directory is specified.</exception>
        public void Reset(DirectoryInfo baseDirectory)
        {
            if (baseDirectory != null)
            {
                if (!baseDirectory.Exists)
                {
                    throw new ArgumentException("Specified base directory doesn't exist.");
                }
                baseDirFull = baseDirectory.FullName;
                
                // Strip off trailing \'s
                if (baseDirFull.EndsWith(Path.DirectorySeparatorChar + "") ||
                    baseDirFull.EndsWith(Path.AltDirectorySeparatorChar + ""))                
                {
                    if (baseDirFull != Path.GetPathRoot(baseDirFull))
                    {
                        baseDirFull = baseDirFull.Substring(0, baseDirFull.Length - 1);
                    }
                }
            }
            else
            {
                this.baseDirFull = null;
            }

            this.baseDirectory = baseDirectory;            

            overallStats = new StatsResult();
            fileStats.Clear();
            directoryStats.Clear();
            aliasStats.Clear();
            cachedCanonicals.Clear();
        }

        /// <summary>
        /// Generate a report with all the aggregated results.
        /// </summary>
        /// <exception cref="StatsReportingException">If any error occurs.</exception>
        /// <returns>A string containing the complete generated report.</returns>
        public abstract string GenerateReport();
    }
}
