/*
Copyright (c)2005, TopCoder, Inc. All rights reserved
Author Cucu
*/
using TopCoder.File.Statistics.Matcher;
using NUnit.Framework;
using TopCoder.File.Statistics;
using System.IO;
using System.Collections;
using TopCoder.Util.ConfigurationManager;
using TopCoder.File.Statistics.AccuracyTests;

namespace TopCoder.File.Statistics.AccuracyTests.Matcher
{
    /// <summary>
    /// Accuracy tests for ExtensionMatcher.
    /// </summary>
    [TestFixture]
    public class ExtensionMatcherAccuracyTest
    {       

        /// <summary>
        /// Set up for each test.
        /// </summary>
        [SetUp]
        public void SetUp()
        {
            TestHelper.LoadConfig(TestHelper.MATCHERS_CONFIG);
        }

        /// <summary>
        /// Tear down for each test.
        /// </summary>
        [TearDown]
        public void Teardown()
        {
            TestHelper.UnloadConfig(TestHelper.MATCHERS_CONFIG);
        }

        /// <summary>
        /// Test the Match method against a single extension.
        /// It test with many different filenames, covering corner cases.
        /// </summary>
        [Test]
        public void MatchWithStringConstructor()
        {
            IFileMatcher matcher = new ExtensionMatcher("txt");
            
            Assert.IsTrue(matcher.Match(new FileInfo(@"file.txt")), @"file.txt should match with extension txt");
            Assert.IsTrue(matcher.Match(new FileInfo(@"file.TxT")), @"file.TxT should match with extension txt");
            Assert.IsTrue(matcher.Match(new FileInfo(@"file.TXT")), @"file.TxT should match with extension txt");
            Assert.IsTrue(matcher.Match(new FileInfo(@".txt")), @".txt should match with extension txt");
            Assert.IsTrue(matcher.Match(new FileInfo(@"c:\documents\version 1.0\hello.txt")), @"c:\documents\version 1.0\hello.txt should match with extension txt");
            Assert.IsTrue(matcher.Match(new FileInfo(@"c:\documents\version 1.0\hello.world.txt")), @"c:\documents\version 1.0\hello.world.txt should match with extension txt");

            Assert.IsFalse(matcher.Match(new FileInfo(@"txt")), @"txt should NOT match with extension txt");
            Assert.IsFalse(matcher.Match(new FileInfo(@"file.txt_")), @"file.txt_ should NOT match with extension txt");
            Assert.IsFalse(matcher.Match(new FileInfo(@"c:\my.txt\hello.bak")), @"c:\my.txt\hello.bak should NOT match with extension txt");
            Assert.IsFalse(matcher.Match(new FileInfo(@"hello.txt.bak")), @"hello.txt.bak should NOT match with extension txt");
        }

        /// <summary>
        /// Test that the list constructor makes a copy of the list
        /// </summary>
        [Test]
        public void TestListConstructor() 
        {
            IList extensions = new ArrayList();
            extensions.Add("txt");
            
            IFileMatcher matcher = new ExtensionMatcher(extensions);
            
            // this shouldn't affect the matcher
            extensions.Add("bak");
            
            Assert.IsFalse(matcher.Match(new FileInfo(@"file.bak")), @"constructor is not making a copy");
        }

        
        /// <summary>
        /// Test the Match method against many extensions, using the constructor that takes a list of extensions.
        /// </summary>
        [Test]
        public void MatchWithListConstructor()
        {
            IList extensions = new ArrayList();
            extensions.Add("cPp");
            extensions.Add("cs");
            extensions.Add("JAVA");
            extensions.Add("pas");
            
            IFileMatcher matcher = new ExtensionMatcher(extensions);

            checkExtensions(matcher);
        }

        /// <summary>
        /// Test the Match method against many extensions, using the constructor that reads from the configuration file.
        /// </summary>
        [Test]
        public void MatchWithConfigManagerConstructor()
        {
            IFileMatcher matcher = new ExtensionMatcher(TestHelper.FILE_STATISTICS_NAMESPACE, "extension.matcher.");
            checkExtensions(matcher);
        }

        /// <summary>
        /// Check that the matcher passed is matching well with the extensions: cpp, cs, java and pas.
        /// </summary>
        /// <param name="matcher">the matcher to check.</param>
        private void checkExtensions(IFileMatcher matcher) {
            Assert.IsTrue(matcher.Match(new FileInfo(@"HelloWorld.cpp")), @"HelloWorld.cpp should match with the extensions");
            Assert.IsTrue(matcher.Match(new FileInfo(@"HelloWorld.cs")), @"HelloWorld.cs should match with the extensions");
            Assert.IsTrue(matcher.Match(new FileInfo(@"HelloWorld.cS")), @"HelloWorld.cS should match with the extensions");
            Assert.IsTrue(matcher.Match(new FileInfo(@"HelloWorld.java")), @"HelloWorld.java should match with the extensions");
            Assert.IsTrue(matcher.Match(new FileInfo(@"HelloWorld.pas")), @"HelloWorld.pas should match with the extensions");
            Assert.IsTrue(matcher.Match(new FileInfo(@"c:\HelloWorld.pas")), @"c:\HelloWorld.pas should match with the extensions");
            Assert.IsTrue(matcher.Match(new FileInfo(@"c:\doc.java\HelloWorld.pas")), @"c:\doc.java\HelloWorld.pas should match with the extensions");

            Assert.IsFalse(matcher.Match(new FileInfo(@"HelloWorld.~cpp")), @"HelloWorld.~cpp should NOT match with the extensions");
            Assert.IsFalse(matcher.Match(new FileInfo(@"java")), @"java should NOT match with the extensions");
            Assert.IsFalse(matcher.Match(new FileInfo(@"java.txt")), @"java.txt should NOT match with the extensions");
            Assert.IsFalse(matcher.Match(new FileInfo(@"Hello.cpp.bak")), @"Hello.cpp.bak should NOT match with the extensions");
            Assert.IsFalse(matcher.Match(new FileInfo(@"c:\program.xpas")), @"c:\program.xpas should NOT match with the extensions");

        }

         }    
}