///
/// $(#)ExtensionMatcherTests.java		Sep 6, 2005
/// 
/// Copyright (c) 2005, TopCoder Inc. All rights reserved.
/// </summary>

using System;
using System.IO;
using System.Collections;
using NUnit.Framework;

using TopCoder.File.Statistics.Matcher;
using TopCoder.Util.ConfigurationManager;

namespace TopCoder.File.Statistics.FailureTests.Matcher
{
	/// <summary>
    /// This test fixture contains the failure tests for <see cref="ExtensionMatcher"/> class.
    ///
    /// <author>MPhk</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (C) 2005 TopCoder Inc., All Rights Reserved.</copyright>
    /// </summary>
    [TestFixture]
	public class ExtensionMatcherTests {
	
	    /// <summary>
	    /// The namespace for this test.
	    /// </summary>
	    private const string NAMESPACE = "TopCoder.File.Statistics.Matcher.FailureTests";
	
	    /// <summary>
	    /// The config file for this test.
	    /// </summary>
	    private const string CONFIG_FILE = "../../test_files/failuretests/matcher/config.xml";
	
	    /// <summary>
	    /// An ExtensionMatcherTests instance to test.
	    /// </summary>
	    private ExtensionMatcher matcher = null;
	
	    /// <summary>
	    /// Load the configuration before testing.
	    /// </summary>
	    [SetUp]
	    protected void SetUp() {
	        // clear the namespace first
	        ConfigManager configManager = ConfigManager.GetInstance();
	        configManager.RemoveFile(CONFIG_FILE);
	        // load the configuration
	        configManager.LoadFile(CONFIG_FILE);
	        
	        matcher = new ExtensionMatcher("txt");
	    }
	
	    /// <summary>
	    /// Remove the configuration after testing.
	    /// </summary>
	    [TearDown]
	    protected void tearDown() {
	        ConfigManager configManager = ConfigManager.GetInstance();
	        configManager.RemoveFile(NAMESPACE);
	    }
	
	    /// <summary>
	    /// Checks whether the constructor ExtensionMatcher(String) properly throws
	    /// ArgumentNullException for null argument.
	    /// </summary>
	    [Test, ExpectedException(typeof(ArgumentNullException))]
	    public void TestExtensionMatcher1() {
            new ExtensionMatcher((string)null);
	    }
	
	    /// <summary>
	    /// Checks whether the constructor ExtensionMatcher(string, string) properly throws
	    /// ArgumentNullException for null argument.
	    /// </summary>
	    [Test, ExpectedException(typeof(ArgumentNullException))]
	    public void TestExtensionMatcher2() {
            new ExtensionMatcher(null, "prefix.");
	    }
	    
	    /// <summary>
	    /// Checks whether the constructor ExtensionMatcher(string, string) properly throws
	    /// ArgumentException for empty argument.
	    /// </summary>
	    [Test, ExpectedException(typeof(ArgumentException))]
	    public void TestExtensionMatcher3() {
            new ExtensionMatcher("", "prefix.");
	    }
	    
	    /// <summary>
	    /// Checks whether the constructor ExtensionMatcher(string, string) properly throws
	    /// ArgumentNullException for null argument.
	    /// </summary>
	    [Test, ExpectedException(typeof(ArgumentNullException))]
	    public void TestExtensionMatcher4() {
            new ExtensionMatcher("namespace", null);
	    }
	
	    /// <summary>
	    /// Checks whether the constructor ExtensionMatcher(Property) properly throws
	    /// ConfigurationException for invalid property - no values in "extensions" sub-property.
	    /// </summary>
	    [Test, ExpectedException(typeof(ConfigurationException))]
	    public void TestExtensionMatcher5() {
            new ExtensionMatcher(NAMESPACE, "extension_invalid1.");
	    }
	
	    /// <summary>
	    /// Checks whether the constructor ExtensionMatcher(Property) properly throws
	    /// ConfigurationException for invalid property - no "extensions" sub-property.
	    /// </summary>
	    [Test, ExpectedException(typeof(ConfigurationException))]
	    public void TestExtensionMatcher6() {
	        new ExtensionMatcher(NAMESPACE, "extension_invalid2.");
	    }
	
	    /// <summary>
	    /// Checks whether the constructor ExtensionMatcher(List) properly throws
	    /// ArgumentNullException for null argument.
	    /// </summary>
	    [Test, ExpectedException(typeof(ArgumentNullException))]
	    public void TestExtensionMatcher7() {
            new ExtensionMatcher((IList)null);
	    }
	
	    /// <summary>
	    /// Checks whether the constructor ExtensionMatcher(List) properly throws
	    /// ArgumentException for empty List argument.
	    /// </summary>
	    [Test, ExpectedException(typeof(ArgumentException))]
	    public void TestExtensionMatcher8() {
            new ExtensionMatcher(new ArrayList());
	    }
	    
	    /// <summary>
	    /// Checks whether the constructor ExtensionMatcher(List) properly throws
	    /// ArgumentException for List argument containing nulls.
	    /// </summary>
	    [Test, ExpectedException(typeof(ArgumentException))]
	    public void TestExtensionMatcher9() {
	        IList list = new ArrayList();
	        list.Add("");
	        list.Add("2");
	        list.Add(null);
	        list.Add("4");

	        new ExtensionMatcher(list);
	    }
	    
	    /// <summary>
	    /// Checks whether the constructor ExtensionMatcher(List) properly throws
	    /// ArgumentException for List argument containing non String elements.
	    /// </summary>
	    [Test, ExpectedException(typeof(ArgumentException))]
	    public void TestExtensionMatcher10() {
	        IList list = new ArrayList();
	        list.Add("1");
	        list.Add(new Object());
	        list.Add("4");

	        new ExtensionMatcher(list);
	    }
	
	    /// <summary>
	    /// Tests the match method with null argument. ArgumentNullException should be thrown.
	    /// </summary>
	    [Test, ExpectedException(typeof(ArgumentNullException))]
	    public void TestMatch() {
            matcher.Match((FileInfo) null);
	    }
	}
}
