///
/// $(#)XmlStatsReportingTests.cs		Sep 6, 2005
/// 
/// Copyright (c) 2005, TopCoder Inc. All rights reserved.
/// </summary>

using System;
using System.IO;
using System.Collections;
using NUnit.Framework;

using TopCoder.File.Statistics.Matcher;
using TopCoder.File.Statistics.Reporting;
using TopCoder.File.Statistics.Processor;
using TopCoder.File.Statistics.Processor.LineCounter;
using TopCoder.Util.ConfigurationManager;

namespace TopCoder.File.Statistics.FailureTests.Reporting
{
	/// <summary>
    /// This test fixture contains the failure tests for <see cref="XmlStatsReportingTests"/> class.
    ///
    /// <author>MPhk</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (C) 2005 TopCoder Inc., All Rights Reserved.</copyright>
    /// </summary>
    [TestFixture]
	public class XmlStatsReportingTests {
	
    	/// <summary>
	    /// The parent base path of the test files.
	    /// </summary>
	    private const string PARENT_BASE_PATH = "../../test_files/failuretests";
    	
	    /// <summary>
	    /// The base path of the test files.
	    /// </summary>
	    private const string BASE_PATH = "../../test_files/failuretests/reporting";
	
	    /// <summary>
	    /// The namespace for this test.
	    /// </summary>
	    private const string NAMESPACE = "TopCoder.File.Statistics.Reporting.FailureTests";
	
	    /// <summary>
	    /// The config file for this test.
	    /// </summary>
	    private const string CONFIG_FILE = "../../test_files/failuretests/reporting/config.xml";
	
	    /// <summary>
	    /// An XmlStatsReporting instance to test.
	    /// </summary>
	    private XmlStatsReporting reporting = null;
	
	    /// <summary>
	    /// A Metric instance to test.
	    /// </summary>
	    private Metric metric1 = new Metric("metric1", "metric1", "foo");
	
	    /// <summary>
	    /// A Metric instance to test.
	    /// </summary>
	    private Metric metric2 = new Metric("metric2", "metric2", "foo");
	
	    /// <summary>
	    /// A StatsResult instance to test.
	    /// </summary>
	    private StatsResult result = null;
	
	    /// <summary>
	    /// A FileProcessor instance to test.
	    /// </summary>
	    private TextLineCounter processor = new TextLineCounter("text", new ExtensionMatcher("txt"));
	
	    /// <summary>
	    /// Sets up this test.
	    /// </summary>
	    [SetUp]
	    protected void SetUp() {
	        reporting = new XmlStatsReporting();
	        result = new StatsResult();
	        result[metric1] = 10;
	        result[metric2] = 20;
	
	        // clear the namespace first
	        ConfigManager configManager = ConfigManager.GetInstance();
	        configManager.RemoveFile(CONFIG_FILE);
	        // load the configuration
	        configManager.LoadFile(CONFIG_FILE);
	    }
	
	    /// <summary>
	    /// Tears down this test.
	    /// </summary>
	    [TearDown]
	    protected void TearDown() {
	        ConfigManager configManager = ConfigManager.GetInstance();
	        configManager.RemoveFile(CONFIG_FILE);
	    }
	
	    /// <summary>
	    /// Checks whether the constructor XmlStatsReporting(string, string) properly throws
	    /// ArgumentNullException for null argument.
	    /// </summary>
	    [Test, ExpectedException(typeof(ArgumentNullException))]
	    public void TestXmlStatsReporting1() {
            new XmlStatsReporting(null, "prefix.");
	    }
	    
	    /// <summary>
	    /// Checks whether the constructor XmlStatsReporting(string, string) properly throws
	    /// ArgumentException for empty argument.
	    /// </summary>
	    [Test, ExpectedException(typeof(ArgumentException))]
	    public void TestXmlStatsReporting2() {
            new XmlStatsReporting("", "prefix.");
	    }
	    
	    /// <summary>
	    /// Checks whether the constructor XmlStatsReporting(string, string) properly throws
	    /// ArgumentNullException for null argument.
	    /// </summary>
	    [Test, ExpectedException(typeof(ArgumentNullException))]
	    public void TestXmlStatsReporting3() {
            new XmlStatsReporting("namespace", null);
	    }
	
	    /// <summary>
	    /// Checks whether the constructor XmlStatsReporting(Property) properly throws
	    /// ConfigurationException for invalid property.
	    /// </summary>
	    [Test, ExpectedException(typeof(ConfigurationException))]
	    public void TestXmlStatsReporting4() {
            new XmlStatsReporting(NAMESPACE, "invalid1.");
	    }
	
	    /// <summary>
	    /// Checks whether the constructor XmlStatsReporting(Property) properly throws
	    /// ConfigurationException for invalid property.
	    /// </summary>
	    [Test, ExpectedException(typeof(ConfigurationException))]
	    public void TestXmlStatsReporting5() {
            new XmlStatsReporting(NAMESPACE, "invalid2.");
	    }
	
	    /// <summary>
	    /// Checks whether the constructor XmlStatsReporting(File) properly throws
	    /// ArgumentNullException for null argument.
	    ///
	    /// @ in case anything goes wrong
	    /// </summary>
	    [Test, ExpectedException(typeof(ArgumentNullException))]
	    public void TestXmlStatsReporting6() {
            new XmlStatsReporting((FileInfo) null);
	    }
	
	    /// <summary>
	    /// Checks whether the constructor XmlStatsReporting(File) properly throws
	    /// ArgumentException for invalid file argument.
	    /// </summary>
	    [Test, ExpectedException(typeof(ArgumentException))]
	    public void TestXmlStatsReporting7() {
            new XmlStatsReporting(new FileInfo(BASE_PATH));
	    }
	
	    /// <summary>
	    /// Checks whether the constructor XmlStatsReporting(File) properly throws
	    /// ArgumentException for invalid file argument.
	    /// </summary>
	    [Test, ExpectedException(typeof(ArgumentException))]
	    public void TestXmlStatsReporting8() {
            new XmlStatsReporting(new FileInfo(BASE_PATH + "/inexistent_file"));
	    }
	
	    /// <summary>
	    /// Checks whether the method aggregate(FileProcessor, File, StatsResult) properly throws
	    /// ArgumentNullException for null argument.
	    /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
	    public void TestAggregate1() {
        	reporting.Aggregate(null, new FileInfo("file"), new StatsResult());
	    }
	
	    /// <summary>
	    /// Checks whether the method aggregate(FileProcessor, File, StatsResult) properly throws
	    /// ArgumentNullException for null argument.
	    /// </summary>
	    [Test, ExpectedException(typeof(ArgumentNullException))]
	    public void TestAggregate2() {
            reporting.Aggregate(processor, null, result);
	    }
	
	    /// <summary>
	    /// Checks whether the method aggregate(FileProcessor, File, StatsResult) properly throws
	    /// ArgumentNullException for null argument.
	    /// </summary>
	    [Test, ExpectedException(typeof(ArgumentNullException))]
	    public void TestAggregate3() {
            reporting.Aggregate(processor, new FileInfo("file"), null);
	    }
	
	    /// <summary>
	    /// Checks whether the method aggregate(FileProcessor, File, StatsResult) properly throws
	    /// StatsReportingException for invalid file argument.
	    /// </summary>
	    [Test, ExpectedException(typeof(StatsReportingException))]
	    public void TestAggregate4() {
	        reporting.Reset(new DirectoryInfo(BASE_PATH));
            reporting.Aggregate(processor, new FileInfo(BASE_PATH), result);
	    }
	
	    /// <summary>
	    /// Checks whether the method aggregate(FileProcessor, File, StatsResult) properly throws
	    /// StatsReportingException for invalid file argument.
	    /// </summary>
	    [Test, ExpectedException(typeof(StatsReportingException))]
	    public void TestAggregate5() {
	        reporting.Reset(new DirectoryInfo(BASE_PATH));
            reporting.Aggregate(processor, new FileInfo(BASE_PATH + "invalid_file"), result);
	    }
	
	    /// <summary>
	    /// Checks whether the method aggregate(FileProcessor, File, StatsResult) properly throws
	    /// StatsReportingException for invalid file argument.
	    /// </summary>
	    [Test, ExpectedException(typeof(StatsReportingException))]
	    public void TestAggregate6() {
	        reporting.Reset(new DirectoryInfo(BASE_PATH));
            reporting.Aggregate(processor, new FileInfo(new DirectoryInfo(PARENT_BASE_PATH) + "/file.txt"), result);
	    }
	
	    /// <summary>
	    /// Checks whether the method reset(File) properly throws
	    /// ArgumentException for invalid folder argument.
	    /// </summary>
	    [Test, ExpectedException(typeof(ArgumentException))]
	    public void TestReset() {
            DirectoryInfo baseDir = new DirectoryInfo("INVALID_DIR");
            reporting.Reset(baseDir);
	    }
    }
}
